# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/services/portfolio-service/portfolio_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.portfolio import create_portfolio_request_pb2 as fintekkers_dot_requests_dot_portfolio_dot_create__portfolio__request__pb2
from fintekkers.requests.portfolio import create_portfolio_response_pb2 as fintekkers_dot_requests_dot_portfolio_dot_create__portfolio__response__pb2
from fintekkers.requests.portfolio import query_portfolio_request_pb2 as fintekkers_dot_requests_dot_portfolio_dot_query__portfolio__request__pb2
from fintekkers.requests.portfolio import query_portfolio_response_pb2 as fintekkers_dot_requests_dot_portfolio_dot_query__portfolio__response__pb2
from fintekkers.requests.util.errors import summary_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=fintekkers/services/portfolio-service/portfolio_service.proto\x12%fintekkers.services.portfolio_service\x1a<fintekkers/requests/portfolio/create_portfolio_request.proto\x1a=fintekkers/requests/portfolio/create_portfolio_response.proto\x1a;fintekkers/requests/portfolio/query_portfolio_request.proto\x1a<fintekkers/requests/portfolio/query_portfolio_response.proto\x1a-fintekkers/requests/util/errors/summary.proto2\xab\x06\n\tPortfolio\x12\x89\x01\n\x0e\x43reateOrUpdate\x12:.fintekkers.requests.portfolio.CreatePortfolioRequestProto\x1a;.fintekkers.requests.portfolio.CreatePortfolioResponseProto\x12\x81\x01\n\x08GetByIds\x12\x39.fintekkers.requests.portfolio.QueryPortfolioRequestProto\x1a:.fintekkers.requests.portfolio.QueryPortfolioResponseProto\x12\x81\x01\n\x06Search\x12\x39.fintekkers.requests.portfolio.QueryPortfolioRequestProto\x1a:.fintekkers.requests.portfolio.QueryPortfolioResponseProto0\x01\x12\x80\x01\n\x07ListIds\x12\x39.fintekkers.requests.portfolio.QueryPortfolioRequestProto\x1a:.fintekkers.requests.portfolio.QueryPortfolioResponseProto\x12\x83\x01\n\x16ValidateCreateOrUpdate\x12:.fintekkers.requests.portfolio.CreatePortfolioRequestProto\x1a-.fintekkers.requests.util.errors.SummaryProto\x12\x80\x01\n\x14ValidateQueryRequest\x12\x39.fintekkers.requests.portfolio.QueryPortfolioRequestProto\x1a-.fintekkers.requests.util.errors.SummaryProtoB\x06\x88\x01\x01\x90\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.services.portfolio_service.portfolio_service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\210\001\001\220\001\001'
  _PORTFOLIO._serialized_start=400
  _PORTFOLIO._serialized_end=1211
_builder.BuildServices(DESCRIPTOR, 'fintekkers.services.portfolio_service.portfolio_service_pb2', globals())
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Message(_message.Message):
    __slots__ = ["message_for_developer", "message_for_user"]
    MESSAGE_FOR_DEVELOPER_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FOR_USER_FIELD_NUMBER: _ClassVar[int]
    message_for_developer: str
    message_for_user: str
    def __init__(self, message_for_user: _Optional[str] = ..., message_for_developer: _Optional[str] = ...) -> None: ...



from fintekkers.models.security import security_pb2 as _security_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateSecurityRequestProto(_message.Message):
    __slots__ = ["object_class", "security_input", "version"]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    SECURITY_INPUT_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    security_input: _security_pb2.SecurityProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., security_input: _Optional[_Union[_security_pb2.SecurityProto, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/position/measure.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(fintekkers/models/position/measure.proto\x12\x1a\x66intekkers.models.position*\xaa\x01\n\x0cMeasureProto\x12\x13\n\x0fUNKNOWN_MEASURE\x10\x00\x12\x15\n\x11\x44IRECTED_QUANTITY\x10\x01\x12\x10\n\x0cMARKET_VALUE\x10\x02\x12\x19\n\x15UNADJUSTED_COST_BASIS\x10\x03\x12\x17\n\x13\x41\x44JUSTED_COST_BASIS\x10\x04\x12\x11\n\rCURRENT_YIELD\x10\x05\x12\x15\n\x11YIELD_TO_MATURITY\x10\x07\x42\x11\x42\rMeasureProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.position.measure_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\rMeasureProtosP\001'
  _MEASUREPROTO._serialized_start=73
  _MEASUREPROTO._serialized_end=243
# @@protoc_insertion_point(module_scope)



from fintekkers.models.security import tenor_type_pb2 as _tenor_type_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TenorProto(_message.Message):
    __slots__ = ["object_class", "tenor_type", "term_value", "version"]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    TENOR_TYPE_FIELD_NUMBER: _ClassVar[int]
    TERM_VALUE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    tenor_type: _tenor_type_pb2.TenorTypeProto
    term_value: str
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., term_value: _Optional[str] = ..., tenor_type: _Optional[_Union[_tenor_type_pb2.TenorTypeProto, str]] = ...) -> None: ...



from fintekkers.models.position import field_pb2 as _field_pb2
from fintekkers.models.position import measure_pb2 as _measure_pb2
from fintekkers.models.position import position_pb2 as _position_pb2
from fintekkers.models.position import position_filter_pb2 as _position_filter_pb2
from fintekkers.models.util import local_timestamp_pb2 as _local_timestamp_pb2
from fintekkers.requests.util import operation_pb2 as _operation_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryPositionRequestProto(_message.Message):
    __slots__ = ["as_of", "fields", "filter_fields", "measures", "object_class", "operation_type", "position_type", "position_view", "version"]
    AS_OF_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELDS_FIELD_NUMBER: _ClassVar[int]
    MEASURES_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    OPERATION_TYPE_FIELD_NUMBER: _ClassVar[int]
    POSITION_TYPE_FIELD_NUMBER: _ClassVar[int]
    POSITION_VIEW_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    as_of: _local_timestamp_pb2.LocalTimestampProto
    fields: _containers.RepeatedScalarFieldContainer[_field_pb2.FieldProto]
    filter_fields: _position_filter_pb2.PositionFilterProto
    measures: _containers.RepeatedScalarFieldContainer[_measure_pb2.MeasureProto]
    object_class: str
    operation_type: _operation_pb2.RequestOperationTypeProto
    position_type: _position_pb2.PositionTypeProto
    position_view: _position_pb2.PositionViewProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., operation_type: _Optional[_Union[_operation_pb2.RequestOperationTypeProto, str]] = ..., position_type: _Optional[_Union[_position_pb2.PositionTypeProto, str]] = ..., position_view: _Optional[_Union[_position_pb2.PositionViewProto, str]] = ..., fields: _Optional[_Iterable[_Union[_field_pb2.FieldProto, str]]] = ..., measures: _Optional[_Iterable[_Union[_measure_pb2.MeasureProto, str]]] = ..., filter_fields: _Optional[_Union[_position_filter_pb2.PositionFilterProto, _Mapping]] = ..., as_of: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
EXECUTED: PositionStatusProto
HYPOTHETICAL: PositionStatusProto
INTENDED: PositionStatusProto
UNKNOWN: PositionStatusProto

class PositionStatusProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []




from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto
from fintekkers.models.security.identifier.identifier_type_pb2 import IdentifierTypeProto

class Identifier():
    def __init__(self, identifier:IdentifierProto):
        self.proto = identifier

    def __str__(self):
        identifier_type_name = IdentifierTypeProto.DESCRIPTOR.values_by_number[self.proto.identifier_type].name
        return f"{identifier_type_name}:{self.proto.identifier_value}"
    
    def get_identifier_value(self) -> str:
        return self.proto.identifier_value
    
    def get_identifier_type(self) -> IdentifierTypeProto:
        return self.proto.identifier_type


from datetime import date
from google.protobuf.any_pb2 import Any

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_status_pb2 import PositionStatusProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.transaction.transaction_type_pb2 import TransactionTypeProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto

from fintekkers.requests.transaction.create_transaction_request_pb2 import CreateTransactionRequestProto
from fintekkers.requests.transaction.query_transaction_request_pb2 import QueryTransactionRequestProto

from fintekkers.wrappers.models.transaction import Transaction
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil

from datetime import datetime

class CreateTransactionRequest():
    @staticmethod
    def create_transaction_request(
        security:SecurityProto=None, portfolio:PortfolioProto=None, \
        trade_date:date=date.today(), settlement_date:date=date.today(), \
        position_status:PositionStatusProto=PositionStatusProto.INTENDED, \
        transaction_type:TransactionTypeProto=TransactionTypeProto.BUY, \
        price:float=-100.00, quantity=100, as_of=datetime.now()
    ):
        '''
            Creates a request to create a transaction

                Parameters:
                        security (SecurityProto): A security proto, will be none if not provided (which would likely fail the creation request)
                        trade_date (date): The trade date, a.k.a the spot date
                        settlement_date (date): Settlement date of the trade
                        position_status (PositionStatusProto) The position status of the transaction, will default to INTENDED
                        transaction_type (TransactionTypeProto): The type of transaction, will default to BUY
                        price (float): Will default to 100 if not provided

                Returns:
                        request (CreateTransactionRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        '''
        transaction:Transaction = Transaction.create_from(security=security, portfolio=portfolio, trade_date=trade_date, \
                                settlement_date=settlement_date, position_status=position_status, \
                                    transaction_type=transaction_type, price=price, quantity=quantity, as_of=as_of)
        
        proto:CreateTransactionRequestProto = CreateTransactionRequestProto(
            create_transaction_input=transaction.proto
        )
        return CreateTransactionRequest(proto=proto)
    
    def __init__(self, proto:CreateTransactionRequestProto):
        self.proto = proto

class QueryTransactionRequest():
    @staticmethod
    def create_query_request(fields:dict):
        '''
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (CreateTransactionRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        '''

        filters = []

        for field in fields:
            field:FieldProto

            field_value = fields[field]            

            packed_value:Any = Any()
            packed_value.Pack(msg=field_value)

            entry = FieldMapEntry(
                field=field, field_value_packed=packed_value
            )

            filters.append(entry)
        
        as_of_proto:LocalTimestampProto = ProtoSerializationUtil.serialize(datetime.now())

        query_request:QueryTransactionRequestProto = QueryTransactionRequestProto(
            search_transaction_input=PositionFilterProto(
                filters=filters
            ),
            as_of=as_of_proto
        )

        return query_request
    
    def __init__(self, proto:QueryTransactionRequestProto):
        self.proto = proto


from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

BUY: TransactionTypeProto
DEPOSIT: TransactionTypeProto
DESCRIPTOR: _descriptor.FileDescriptor
MATURATION: TransactionTypeProto
MATURATION_OFFSET: TransactionTypeProto
SELL: TransactionTypeProto
UNKNOWN: TransactionTypeProto
WITHDRAWAL: TransactionTypeProto

class TransactionTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/util/local_timestamp.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,fintekkers/models/util/local_timestamp.proto\x12\x16\x66intekkers.models.util\x1a\x1fgoogle/protobuf/timestamp.proto\"W\n\x13LocalTimestampProto\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\ttime_zone\x18\x02 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.util.local_timestamp_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LOCALTIMESTAMPPROTO._serialized_start=105
  _LOCALTIMESTAMPPROTO._serialized_end=192
# @@protoc_insertion_point(module_scope)



from datetime import date
from google.protobuf.any_pb2 import Any

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_status_pb2 import PositionStatusProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.transaction.transaction_type_pb2 import TransactionTypeProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.requests.portfolio.query_portfolio_request_pb2 import (
    QueryPortfolioRequestProto,
)

from fintekkers.requests.transaction.create_transaction_request_pb2 import (
    CreateTransactionRequestProto,
)
from fintekkers.requests.transaction.query_transaction_request_pb2 import (
    QueryTransactionRequestProto,
)

from fintekkers.wrappers.models.transaction import Transaction
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil

from datetime import datetime


class CreatePortfolioRequest:
    @staticmethod
    def create_transaction_request(
        security: SecurityProto = None,
        portfolio: PortfolioProto = None,
        trade_date: date = date.today(),
        settlement_date: date = date.today(),
        position_status: PositionStatusProto = PositionStatusProto.INTENDED,
        transaction_type: TransactionTypeProto = TransactionTypeProto.BUY,
        price: float = -100.00,
        quantity=100,
        as_of=datetime.now(),
    ):
        """
        Creates a request to create a transaction

            Parameters:
                    security (SecurityProto): A security proto, will be none if not provided (which would likely fail the creation request)
                    trade_date (date): The trade date, a.k.a the spot date
                    settlement_date (date): Settlement date of the trade
                    position_status (PositionStatusProto) The position status of the transaction, will default to INTENDED
                    transaction_type (TransactionTypeProto): The type of transaction, will default to BUY
                    price (float): Will default to 100 if not provided

            Returns:
                    request (CreateTransactionRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """
        transaction: Transaction = Transaction.create_from(
            security=security,
            portfolio=portfolio,
            trade_date=trade_date,
            settlement_date=settlement_date,
            position_status=position_status,
            transaction_type=transaction_type,
            price=price,
            quantity=quantity,
            as_of=as_of,
        )

        proto: CreateTransactionRequestProto = CreateTransactionRequestProto(
            create_transaction_input=transaction.proto
        )
        return CreatePortfolioRequest(proto=proto)

    def __init__(self, proto: CreateTransactionRequestProto):
        self.proto = proto


class QueryPortfolioRequest:
    @staticmethod
    def create_query_request(fields: dict):
        """
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (CreateTransactionRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """

        filters = []

        # TODO: This code is duplicated in transaction.py
        for field in fields:
            field: FieldProto

            field_value = fields[field]
            entry = None

            if field_value.__class__ == str:
                entry = FieldMapEntry(field=field, string_value=field_value)
            else:
                packed_value = Any()
                packed_value.Pack(msg=field_value)

                entry = FieldMapEntry(field=field, field_value_packed=packed_value)

            filters.append(entry)

        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        query_request: QueryPortfolioRequestProto = QueryPortfolioRequestProto(
            search_portfolio_input=PositionFilterProto(filters=filters),
            as_of=as_of_proto,
        )

        return QueryPortfolioRequest(query_request)

    def __init__(self, proto: QueryPortfolioRequestProto):
        self.proto = proto



from fintekkers.models.portfolio import portfolio_pb2 as _portfolio_pb2
from fintekkers.requests.portfolio import query_portfolio_request_pb2 as _query_portfolio_request_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryPortfolioResponseProto(_message.Message):
    __slots__ = ["object_class", "portfolio_response", "query_portfolio_request", "version"]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    PORTFOLIO_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    QUERY_PORTFOLIO_REQUEST_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    portfolio_response: _containers.RepeatedCompositeFieldContainer[_portfolio_pb2.PortfolioProto]
    query_portfolio_request: _query_portfolio_request_pb2.QueryPortfolioRequestProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., query_portfolio_request: _Optional[_Union[_query_portfolio_request_pb2.QueryPortfolioRequestProto, _Mapping]] = ..., portfolio_response: _Optional[_Iterable[_Union[_portfolio_pb2.PortfolioProto, _Mapping]]] = ...) -> None: ...



from fintekkers.models.security.identifier import identifier_type_pb2 as _identifier_type_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IdentifierProto(_message.Message):
    __slots__ = ["identifier_type", "identifier_value", "object_class", "version"]
    IDENTIFIER_TYPE_FIELD_NUMBER: _ClassVar[int]
    IDENTIFIER_VALUE_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    identifier_type: _identifier_type_pb2.IdentifierTypeProto
    identifier_value: str
    object_class: str
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., identifier_value: _Optional[str] = ..., identifier_type: _Optional[_Union[_identifier_type_pb2.IdentifierTypeProto, str]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

BOND_SECURITY: SecurityTypeProto
CASH_SECURITY: SecurityTypeProto
DESCRIPTOR: _descriptor.FileDescriptor
EQUITY_SECURITY: SecurityTypeProto
FRN: SecurityTypeProto
TIPS: SecurityTypeProto
UNKNOWN_SECURITY_TYPE: SecurityTypeProto

class SecurityTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

CASH: IdentifierTypeProto
CUSIP: IdentifierTypeProto
DESCRIPTOR: _descriptor.FileDescriptor
EXCH_TICKER: IdentifierTypeProto
FIGI: IdentifierTypeProto
ISIN: IdentifierTypeProto
OSI: IdentifierTypeProto
UNKNOWN_IDENTIFIER_TYPE: IdentifierTypeProto

class IdentifierTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from fintekkers.models.strategy import strategy_pb2 as _strategy_pb2
from fintekkers.models.util import local_timestamp_pb2 as _local_timestamp_pb2
from fintekkers.models.util import decimal_value_pb2 as _decimal_value_pb2
from fintekkers.models.util import uuid_pb2 as _uuid_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class MapFieldEntry(_message.Message):
    __slots__ = ["key", "value"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: _strategy_pb2.StrategyProto
    value: _decimal_value_pb2.DecimalValueProto
    def __init__(self, key: _Optional[_Union[_strategy_pb2.StrategyProto, _Mapping]] = ..., value: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ...) -> None: ...

class StrategyAllocationProto(_message.Message):
    __slots__ = ["allocations", "as_of", "is_link", "object_class", "uuid", "version"]
    ALLOCATIONS_FIELD_NUMBER: _ClassVar[int]
    AS_OF_FIELD_NUMBER: _ClassVar[int]
    IS_LINK_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    UUID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    allocations: _containers.RepeatedCompositeFieldContainer[MapFieldEntry]
    as_of: _local_timestamp_pb2.LocalTimestampProto
    is_link: bool
    object_class: str
    uuid: _uuid_pb2.UUIDProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., uuid: _Optional[_Union[_uuid_pb2.UUIDProto, _Mapping]] = ..., as_of: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., is_link: bool = ..., allocations: _Optional[_Iterable[_Union[MapFieldEntry, _Mapping]]] = ...) -> None: ...



import time
from datetime import date, datetime
from uuid import uuid4
from google.protobuf.any_pb2 import Any
from google.protobuf.timestamp_pb2 import Timestamp

from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.security.coupon_frequency_pb2 import NO_COUPON, SEMIANNUALLY
from fintekkers.models.security.coupon_type_pb2 import FIXED, FLOAT, ZERO
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto
from fintekkers.models.security.identifier.identifier_type_pb2 import CUSIP
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.security.security_quantity_type_pb2 import ORIGINAL_FACE_VALUE
from fintekkers.models.security.security_type_pb2 import (
    BOND_SECURITY,
    FRN,
    SecurityTypeProto,
)
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto

from fintekkers.models.security.bond.issuance_pb2 import IssuanceProto

from fintekkers.requests.security.create_security_request_pb2 import (
    CreateSecurityRequestProto,
)
from fintekkers.requests.security.query_security_request_pb2 import (
    QuerySecurityRequestProto,
)

from fintekkers.wrappers.models.security import Security
from fintekkers.wrappers.models.util.date_utils import get_date_proto
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil


class CreateSecurityRequest:
    @staticmethod
    def create_ust_security_request(
        cusip: str,
        cash_security: SecurityProto,
        security_type: SecurityTypeProto = SecurityTypeProto.BOND_SECURITY,
        coupon_rate: float = 0.0,
        spread: float = 0.0,
        face_value: float = 0.0,
        issue_date: date = date.today(),
        dated_date: date = date.today(),
        maturity_date: date = date.today(),
        post_auction_outstanding_quantity: float = 0.0,
        auction_total_accepted: float = 0.0,
        auction_announcement_date: date = None,
    ):
        """
        Creates a request to create a security representing a US treasury (bills, notes and bonds)

            Parameters:
                    Parameters are already in protos, see type hints. Post auction quantity refers to the
                    amount of the security that existed after the auction of this security (for re-issues).
                    Total accepted refers to the amount of bond that was sold at the auction.

            Returns:
                    request (CreateSecurityRequest): A request wrapper, with the fields attached
        """
        id = IdentifierProto(identifier_type=CUSIP, identifier_value=cusip)

        security_type = BOND_SECURITY
        coupon_frequency = SEMIANNUALLY
        coupon_type = FIXED

        # if security_type == TIPS:
        #     security_type = TIPS
        if security_type == FRN:
            # security_type = FRN
            coupon_type = FLOAT
            coupon_rate = spread
        if security_type == ZERO:
            coupon_type = ZERO
            coupon_frequency = NO_COUPON

        issue_date_proto = get_date_proto(issue_date)
        dated_date_proto = get_date_proto(dated_date) if dated_date != None else None
        maturity_date_proto = get_date_proto(maturity_date)

        timstamp_seconds = int(time.mktime(issue_date.timetuple()))

        issuance_list = []

        if auction_announcement_date is not None:
            issuance = IssuanceProto(
                as_of=LocalTimestampProto(
                    time_zone="America/New_York",
                    timestamp=Timestamp(seconds=timstamp_seconds, nanos=0),
                ),
                version="0.0.1",
                auction_announcement_date=ProtoSerializationUtil.serialize(
                    auction_announcement_date
                ),
                total_accepted=ProtoSerializationUtil.serialize(auction_total_accepted),
                post_auction_outstanding_quantity=ProtoSerializationUtil.serialize(
                    post_auction_outstanding_quantity
                ),
            )
            issuance_list.append(issuance)

        security_proto: SecurityProto = SecurityProto(
            as_of=LocalTimestampProto(
                time_zone="America/New_York",
                timestamp=Timestamp(seconds=timstamp_seconds, nanos=0),
            ),
            uuid=UUIDProto(raw_uuid=uuid4().bytes),
            issuer_name="US Government",
            identifier=id,
            issue_date=issue_date_proto,
            dated_date=dated_date_proto,
            maturity_date=maturity_date_proto,
            security_type=security_type,
            quantity_type=ORIGINAL_FACE_VALUE,
            settlement_currency=cash_security,
            coupon_frequency=coupon_frequency,
            coupon_type=coupon_type,
            coupon_rate=ProtoSerializationUtil.serialize(coupon_rate),
            asset_class="Fixed Income",
            face_value=ProtoSerializationUtil.serialize(face_value),
            issuance_info=issuance_list,
        )

        security = Security(security_proto)

        return CreateSecurityRequest.create_or_update_request(security)

    @staticmethod
    def create_or_update_request(security: Security):
        proto: CreateSecurityRequestProto = CreateSecurityRequestProto(
            security_input=security.proto
        )

        return CreateSecurityRequest(proto=proto)

    def __init__(self, proto: CreateSecurityRequestProto):
        self.proto = proto


class QuerySecurityRequest:
    @staticmethod
    def create_query_request(fields: dict):
        """
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (QuerySecurityRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """

        filters = []

        for field in fields:
            field: FieldProto

            field_value = fields[field]

            packed_value: Any = Any()
            packed_value.Pack(msg=field_value)

            entry = FieldMapEntry(field=field, field_value_packed=packed_value)

            filters.append(entry)

        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        request: QuerySecurityRequestProto = QuerySecurityRequestProto(
            search_security_input=PositionFilterProto(filters=filters),
            as_of=as_of_proto,
        )

        return QuerySecurityRequest(proto=request)

    def __init__(self, proto: CreateSecurityRequestProto):
        self.proto = proto



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/transaction/transaction_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4fintekkers/models/transaction/transaction_type.proto\x12\x1d\x66intekkers.models.transaction*z\n\x14TransactionTypeProto\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03\x42UY\x10\x01\x12\x08\n\x04SELL\x10\x02\x12\x0b\n\x07\x44\x45POSIT\x10\x03\x12\x0e\n\nWITHDRAWAL\x10\x04\x12\x0e\n\nMATURATION\x10\x05\x12\x15\n\x11MATURATION_OFFSET\x10\x06\x42\x19\x42\x15TransactionTypeProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.transaction.transaction_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\025TransactionTypeProtosP\001'
  _TRANSACTIONTYPEPROTO._serialized_start=87
  _TRANSACTIONTYPEPROTO._serialized_end=209
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fintekkers/models/security/security_quantity_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7fintekkers/models/security/security_quantity_type.proto\x12\x1a\x66intekkers.models.security*h\n\x19SecurityQuantityTypeProto\x12\x19\n\x15UNKNOWN_QUANTITY_TYPE\x10\x00\x12\x17\n\x13ORIGINAL_FACE_VALUE\x10\x01\x12\x0c\n\x08NOTIONAL\x10\x02\x12\t\n\x05UNITS\x10\x03\x42\x1e\x42\x1aSecurityQuantityTypeProtosP\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.security.security_quantity_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\032SecurityQuantityTypeProtosP\001'
  _SECURITYQUANTITYTYPEPROTO._serialized_start=87
  _SECURITYQUANTITYTYPEPROTO._serialized_end=191
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

ADJUSTED_COST_BASIS: MeasureProto
CURRENT_YIELD: MeasureProto
DESCRIPTOR: _descriptor.FileDescriptor
DIRECTED_QUANTITY: MeasureProto
MARKET_VALUE: MeasureProto
UNADJUSTED_COST_BASIS: MeasureProto
UNKNOWN_MEASURE: MeasureProto
YIELD_TO_MATURITY: MeasureProto

class MeasureProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from fintekkers.models.position import position_util_pb2 as _position_util_pb2
from fintekkers.requests.valuation import valuation_request_pb2 as _valuation_request_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ValuationResponseProto(_message.Message):
    __slots__ = ["measure_results", "object_class", "valuation_request", "version"]
    MEASURE_RESULTS_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VALUATION_REQUEST_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    measure_results: _containers.RepeatedCompositeFieldContainer[_position_util_pb2.MeasureMapEntry]
    object_class: str
    valuation_request: _valuation_request_pb2.ValuationRequestProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., valuation_request: _Optional[_Union[_valuation_request_pb2.ValuationRequestProto, _Mapping]] = ..., measure_results: _Optional[_Iterable[_Union[_position_util_pb2.MeasureMapEntry, _Mapping]]] = ...) -> None: ...



import grpc

#Insure local channel
LOCAL_CHANNEL = grpc.insecure_channel('127.0.0.1:8082')
#Secure channel using SSL with default credentials
PROD_CHANNEL = grpc.secure_channel('api.fintekkers.org:8082', grpc.ssl_channel_credentials())
CHANNEL = LOCAL_CHANNEL

from grpc import _channel

def get_channel() -> _channel.Channel:
    return CHANNEL




from datetime import date, datetime
from uuid import uuid4
from google.protobuf.timestamp_pb2 import Timestamp

from fintekkers.models.transaction.transaction_pb2 import TransactionProto
from fintekkers.models.transaction.transaction_type_pb2 import TransactionTypeProto

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.position.position_status_pb2 import PositionStatusProto

from fintekkers.models.price.price_pb2 import PriceProto
from fintekkers.models.security.security_pb2 import SecurityProto

from fintekkers.models.util.local_date_pb2 import LocalDateProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.models.util.decimal_value_pb2 import DecimalValueProto
from fintekkers.wrappers.models.util.date_utils import get_date_from_proto

class Transaction():
    @staticmethod
    def create_from(
        security:SecurityProto=None, portfolio:PortfolioProto=None, \
        trade_date:date=date.today(), settlement_date:date=date.today(), \
        position_status:PositionStatusProto=PositionStatusProto.INTENDED, \
        transaction_type:TransactionTypeProto=TransactionTypeProto.BUY, \
        price:float=-100.00, quantity=100, 
        as_of:datetime=datetime.now()):
        
        as_of_proto = LocalTimestampProto(timestamp=Timestamp(seconds=int(get_date_from_proto(as_of).timestamp()), nanos=0), time_zone="America/New_York")

        return Transaction(proto=TransactionProto(
            as_of=as_of_proto,
            is_cancelled=False,
            is_link=False,
            object_class="Transaction",
            portfolio=portfolio,
            security=security,
            position_status=position_status,
            price=PriceProto(
                uuid=UUIDProto(raw_uuid=uuid4().bytes),
                as_of=as_of_proto,
                price=DecimalValueProto(arbitrary_precision_value=f"{price}"),
                security=security
            ),
            transaction_type=transaction_type,
            quantity=DecimalValueProto(arbitrary_precision_value=f"{quantity}"),
            trade_date=LocalDateProto(year=trade_date.year, month=trade_date.month, day=trade_date.day),
            settlement_date=LocalDateProto(year=settlement_date.year, month=settlement_date.month, day=settlement_date.day),
            uuid=UUIDProto(raw_uuid=uuid4().bytes),
            trade_name="No trade name",
            strategy_allocation=None
        ))

    def __init__(self, proto:TransactionProto):
        self.proto:TransactionProto = proto

class TransactionType():
    def __init__(self, proto: TransactionTypeProto):
        self.proto = proto

    def __str__(self) -> str:
        return TransactionTypeProto.Name(self.proto)



from fintekkers.models.security.bond import auction_type_pb2 as _auction_type_pb2
from fintekkers.models.util import decimal_value_pb2 as _decimal_value_pb2
from fintekkers.models.util import local_date_pb2 as _local_date_pb2
from fintekkers.models.util import local_timestamp_pb2 as _local_timestamp_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class IssuanceProto(_message.Message):
    __slots__ = ["as_of", "auction_announcement_date", "auction_issue_date", "auction_offering_amount", "auction_type", "mature_security_amount", "object_class", "post_auction_outstanding_quantity", "price_for_single_price_auction", "total_accepted", "valid_from", "valid_to", "version"]
    AS_OF_FIELD_NUMBER: _ClassVar[int]
    AUCTION_ANNOUNCEMENT_DATE_FIELD_NUMBER: _ClassVar[int]
    AUCTION_ISSUE_DATE_FIELD_NUMBER: _ClassVar[int]
    AUCTION_OFFERING_AMOUNT_FIELD_NUMBER: _ClassVar[int]
    AUCTION_TYPE_FIELD_NUMBER: _ClassVar[int]
    MATURE_SECURITY_AMOUNT_FIELD_NUMBER: _ClassVar[int]
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    POST_AUCTION_OUTSTANDING_QUANTITY_FIELD_NUMBER: _ClassVar[int]
    PRICE_FOR_SINGLE_PRICE_AUCTION_FIELD_NUMBER: _ClassVar[int]
    TOTAL_ACCEPTED_FIELD_NUMBER: _ClassVar[int]
    VALID_FROM_FIELD_NUMBER: _ClassVar[int]
    VALID_TO_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    as_of: _local_timestamp_pb2.LocalTimestampProto
    auction_announcement_date: _local_date_pb2.LocalDateProto
    auction_issue_date: _local_date_pb2.LocalDateProto
    auction_offering_amount: _decimal_value_pb2.DecimalValueProto
    auction_type: _auction_type_pb2.AuctionTypeProto
    mature_security_amount: _decimal_value_pb2.DecimalValueProto
    object_class: str
    post_auction_outstanding_quantity: _decimal_value_pb2.DecimalValueProto
    price_for_single_price_auction: _decimal_value_pb2.DecimalValueProto
    total_accepted: _decimal_value_pb2.DecimalValueProto
    valid_from: _local_timestamp_pb2.LocalTimestampProto
    valid_to: _local_timestamp_pb2.LocalTimestampProto
    version: str
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., as_of: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., valid_from: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., valid_to: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., auction_announcement_date: _Optional[_Union[_local_date_pb2.LocalDateProto, _Mapping]] = ..., auction_issue_date: _Optional[_Union[_local_date_pb2.LocalDateProto, _Mapping]] = ..., post_auction_outstanding_quantity: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ..., auction_offering_amount: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ..., auction_type: _Optional[_Union[_auction_type_pb2.AuctionTypeProto, str]] = ..., price_for_single_price_auction: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ..., total_accepted: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ..., mature_security_amount: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ...) -> None: ...
